#!/usr/bin/python
# -*- coding:utf8 -*-

# Codé par 20-100 (commencé le 23/04/12)

""" Un bot IRC destiné à s'interfacer avec la Note Kfet 2015 """

import threading
import random
import time
import json
import re
import os
import signal
import sys

# Oui, j'ai recodé ma version d'irclib pour pouvoir rattrapper les SIGHUP
sys.path.insert(0, "/home/vincent/scripts/python-myirclib")
import irclib
# On veut réagir sur la partie du whois qui dit qu'un nick est registered
irclib.numeric_events['307'] = "whoisregnick"
import ircbot

from commands import getstatusoutput as ex

#: Config de basile
import config
#: Module responsable du dialogue avec la NoteKfet2015
import nk
#: Module de réponse aux questions de base
import isit
#: Module définissant les erreurs
import errors
#: Module de gestion des utilisateurs
import users

# la partie qui réfère au fichier lui-même est mieux ici
# sinon on réfère la config et pas le fichier lui-même
config.thisfile = os.path.realpath(__file__)

def get_config_logfile(serveur):
    """Renvoie le nom du fichier de log en fonction du ``serveur`` et de la config."""
    serveurs = {"acoeur.crans.org" : "acoeur",
                "irc.crans.org" : "crans"}
    return config.logfile_template % (serveurs[serveur],)

def get_filesize():
    """Récupère la taille de ce fichier."""
    return ex("ls -s %s" % (config.thisfile))[1].split()[0]

def log(serveur, channel, auteur=None, message=None):
    """Enregistre une ligne de log."""
    if auteur == message == None:
        # alors c'est que c'est pas un channel mais juste une ligne de log
        chain = u"%s %s" % (time.strftime("%F %T"), channel)
    else:
        chain = u"%s [%s:%s] %s" % (time.strftime("%F %T"), channel, auteur, message)
    f = open(get_config_logfile(serveur), "a")
    f.write((chain + u"\n").encode("utf-8"))
    f.close()
    if config.debug_stdout:
        print chain.encode("utf-8")

def ignore_event(serv, ev):
    """Retourne ``True`` si il faut ignorer cet évènement."""
    for (blackmask, exceptlist) in config.blacklisted_masks:
        usermask = ev.source()
        blackit = bool(irclib.mask_matches(usermask, blackmask))
        exceptit = any([bool(irclib.mask_matches(usermask, exceptmask)) for exceptmask in exceptlist])
        if exceptit: # Il est exempté
            return False
        else:
            if blackit: # Il n'est pas exempté et matche la blacklist
                return True


def bot_unicode(chain):
    """Essaye de décoder ``chain`` en UTF-8.
       Lève une py:class:`errors.UnicodeBotError` en cas d'échec."""
    try:
        return chain.decode("utf8")
    except UnicodeDecodeError as exc:
        raise errors.UnicodeBotError


class Basile(ircbot.SingleServerIRCBot):
    """Classe principale : définition du bot Basile."""
    def __init__(self, serveur, debug=False):
        temporary_pseudo = config.irc_pseudo + str(random.randrange(10000,100000))
        ircbot.SingleServerIRCBot.__init__(self, [(serveur, 6667)],
                              temporary_pseudo, "Basile, le bot irc. [Codé par 20-100]", 10)
        self.debug = debug
        self.serveur = serveur
        self.overops = config.overops
        self.ops = self.overops + config.ops
        self.report_bugs_to = config.report_bugs_to
        self.chanlist = config.chanlist
        self.stay_channels = config.stay_channels
        self.quiet_channels = config.quiet_channels
        self.last_perdu = 0
        # On charge la base de données d'utilisateurs
        self.users = users.UserDB()
        self.users.load()
    
    ### Communication NK
    def new_connection_NK(self, serv, username, password, typ="bdd"):
        """Renvoie (``True``, <une socket ouverte et authentifiée sur la NoteKfet2015>)
           ou bien (``False``, None)"""
        try:
            login_result, sock = nk.login(username, password, typ)
            info, retcode, errmsg = login_result["msg"], login_result["retcode"], login_result["errmsg"]
        except nk.NKRefused as exc:
            for report in self.report_bugs_to:
                serv.privmsg(report, "Le Serveur NK2015 est down.")
            return (False, None, None)
        except nk.NKHelloFailed as exc:
            for report in self.report_bugs_to:
                serv.privmsg(report,
                             "La version du protocole utilisée n'est pas supportée par le serveur NK2015.")
            return (False, None, None)
        except nk.NKUnknownError as exc:
            erreurs = ["Une fucking erreur inconnue s'est produite"]
            erreurs += str(exc).split("\n")
            for report in self.report_bugs_to:
                for err in erreurs:
                    serv.privmsg(report, err)
            return (False, None, None)
        except Exception as exc:
            # Exception qui ne vient pas de la communication avec le serveur NK2015
            log(self.serveur, "Erreur dans new_connection_NK\n" + str(exc))
            return (False, None, None)
        if retcode == 0:
            return (True, info, sock)
        else:
            return (False, None, None)
    
    ### Utilitaires
    def _getnick(self):
        """Récuère le nick effectif du bot sur le serveur."""
        return self.serv.get_nickname()
    nick = property(_getnick)
    
    def give_me_my_pseudo(self, serv):
        """Récupère le pseudo auprès de NickServ."""
        serv.privmsg("NickServ", "RECOVER %s %s" % (config.irc_pseudo, config.irc_password))
        serv.privmsg("NickServ", "RELEASE %s %s" % (config.irc_pseudo, config.irc_password))
        time.sleep(0.3)
        serv.nick(config.irc_pseudo)
    
    def pourmoi(self, serv, message):
        """Renvoie (False, lemessage) ou (True, le message amputé de "pseudo: ")"""
        pseudo = self.nick
        pseudo = pseudo.decode("utf-8")
        size = len(pseudo)
        if message[:size] == pseudo and len(message) > size and message[size] == ":":
            return (True, message[size+1:].lstrip(" "))
        else:
            return (False, message)
    
    ### Exécution d'actions
    def lost(self, serv, channel, forced=False):
        """Réaction à un trigger de perdu.
           Annonce "J'ai perdu" sur le channel si on n'a pas perdu depuis un certain temps."""
        if self.last_perdu + config.time_between_perdu < time.time() or forced:
            if not channel in self.quiet_channels or forced:
                serv.privmsg(channel, "J'ai perdu !")
            self.last_perdu = time.time()
            delay = config.time_between_perdu_trigger
            delta = config.time_between_perdu_trigger_delta
            serv.execute_delayed(random.randrange(delay - delta, delay + delta), self.lost, (serv, channel))
    
    def quitter(self, chan, leave_message=None):
        """Quitter un channel avec un message customisable."""
        if leave_message == None:
            leave_message = random.choice(config.leave_messages)
        self.serv.part(chan, message=leave_message.encode("utf8"))
    
    def mourir(self):
        """Se déconnecter du serveur IRC avec un message customisable."""
        quit_message = random.choice(config.quit_messages)
        self.die(msg=quit_message.encode("utf8"))
    
    def execute_reload(self, auteur=None):
        """Recharge la config."""
        reload(config)
        isit.regexp_compile()
        if auteur in [None, "SIGHUP"]:
            towrite = "Config reloaded" + " (SIGHUP received)" * (auteur == "SIGHUP")
            for to in config.report_bugs_to:
                self.serv.privmsg(to, towrite)
            log(self.serveur, towrite)
            return True, None
        else:
            return True, u"Config reloaded"
    
    def crash(self, who="nobody", chan="nowhere"):
        """Fait crasher le bot."""
        where = "en privé" if chan == "priv" else "sur le chan %s" % chan
        raise errors.CrashError((u"Crash demandé par %s %s" % (who, where)).encode("utf-8"))
    
    ACTIONS = {
        "reload" : execute_reload,
        }
    
    def execute_something(self, something, params, place=None, auteur=None):
        """Exécute une action et répond son résultat à ``auteur``
           sur un chan ou en privé en fonction de ``place``"""
        action = self.ACTIONS[something]
        success, message = action(self, **params)
        if message:
            if irclib.is_channel(place):
                message = "%s: %s" % (auteur, message.encode("utf-8"))
            self.serv.privmsg(place, message)
        log(self.serveur, place, auteur, something + "%r" % params + ("[successful]" if success else "[failed]"))
    
    def whois(self, pseudo, askedwhy, askedby):
        """Demande un whois sur ``pseudo``. La réponse sera handled par une autre fonction."""
        self.users.pending_whois[pseudo] = ["pending", askedwhy, askedby, None]
        self.serv.whois([pseudo])
    
    ### Surcharge des events du Bot
    def on_welcome(self, serv, ev):
        """À l'arrivée sur le serveur."""
        self.serv = serv # ça serv ira :)
        self.give_me_my_pseudo(serv)
        serv.privmsg("NickServ", "IDENTIFY %s" % (config.irc_password))
        log(self.serveur, "Connected")
        if self.debug:
            self.chanlist = ["#bot"]
        for c in self.chanlist:
            log(self.serveur, "JOIN %s" % (c))
            serv.join(c)
        # on ouvre la connexion note de Basile, special user
        self.nk = self.new_connection_NK(serv, config.note_pseudo, config.note_password, "special")[2]
        if self.nk == None:
            for report in self.report_bugs_to:
                serv.privmsg(report, "Connection to NK2015 failed, invalid password ?")

    def on_whoisregnick(self, serv, ev):
        """Appelée sur une réponse à un whois au moment où ça dit "is a registered nick".
           J'ai vérifié, "is a registered nick" ça inclu le fiat qu'il est identified correctement.
           
           On stocke l'information comme quoi cette personne est registered, et quand c'était."""
        pseudo, phrase = ev.arguments()
        if phrase == 'is a registered nick':
            # Le whois n'est plus "pending"
            if self.users.pending_whois.has_key(pseudo):
                self.users.pending_whois[pseudo][0] = "registered"
                self.users.pending_whois[pseudo][3] = time.time()
                _, askedwhy, askedby, _ = self.users.pending_whois[pseudo]
                if askedwhy == "cmd WHOIS":
                    # Ce whois a été demandé par quelqu'un, on lui répond
                    self.serv.privmsg(askedby, "%s is a registered nick" % (pseudo,))
    
    def on_endofwhois(self, serv, ev):
        """Si on arrive à la fin du whois, on va voir si on n'a pas reçu "is a registered nick"
           c'est que le pseudo n'est pas identifié. """
        pseudo = ev.arguments()[0]
        # On laisse le temps au bot de souffler un coup
        serv.execute_delayed(config.whois_timeout, self.fail_whoisregnick, (pseudo,))

    def fail_whoisregnick(self, pseudo):
        """Maintenant qu'on a laissé quelques secondes au bot pour gérer les affaires courantes,
           on considère que le pseudo n'est pas registered. """
        # Attention, parce qu'il se pourrait qu'on n'ait pas sollicité ce whois
        # et que donc pending_whois n'ai pas été peuplé en conséquence
        if self.users.pending_whois.has_key(pseudo):
            status, askedwhy, askedby, _ = self.users.pending_whois[pseudo]
            if status == "pending":
                # Si le status est encore pending, on n'a pas eu de réponse positive, donc elle est négative
                self.users.pending_whois[pseudo] = "notregistered"
                if askedwhy == "cmd WHOIS":
                    self.serv.privmsg(askedby, "%s is NOT a registered nick" % (pseudo,))
    
    def on_privmsg(self, serv, ev):
        """À la réception d'un message en privé."""
        if ignore_event(serv, ev):
            return
        message = ev.arguments()[0]
        auteur = irclib.nm_to_n(ev.source())
        try:
            message = bot_unicode(message)
        except errors.UnicodeBotError:
            if config.utf8_trigger:
                serv.privmsg(auteur, random.choice(config.utf8_fail_answers).encode("utf8"))
            return
        message = message.split()
        cmd = message[0].lower()
        notunderstood = False
        if cmd == u"help":
            op,overop=auteur in self.ops, auteur in self.overops
            if len(message)==1:
                helpmsg = config.helpmsg_default
                if op:
                    helpmsg += config.helpmsg_ops
                if overop:
                    helpmsg += config.helpmsg_overops
            else:
                helpmsgs = config.helpdico.get(message[1].lower(), ["Commande inconnue.", None, None])
                helpmsg = helpmsgs[0]
                if op and helpmsgs[1]:
                    if helpmsg:
                        helpmsg += "\n" + helpmsgs[1]
                    else:
                        helpmsg = helpmsgs[1]
                if overop and helpmsgs[2]:
                    if helpmsg:
                        helpmsg += "\n" + helpmsgs[2]
                    else:
                        helpmsg = helpmsgs[2]
                if not helpmsg: # Un non-op a demandé de l'aide sur une commande dont il n'est pas censé connaître l'existence
                    helpmsg = "Commande inacessible."
            for ligne in helpmsg.split("\n"):
                serv.privmsg(auteur, ligne.encode("utf-8"))
        elif cmd == u"identify":
            if len(message) == 1:
                if self.users.has(auteur):
                    infos = self.users[auteur].get_infos(self.nk, serv, auteur)
                    serv.privmsg(auteur, (u"Vous avez le compte note n°%(idbde)s, pseudo : %(pseudo)s." % infos
                                     ).encode("utf8"))
                else:
                    serv.privmsg(auteur, "Je ne connais pas votre note.")
            elif len(message) >= 3:
                username, password = message[1], " ".join(message[2:])
                success, info, _ = self.new_connection_NK(serv, username, password)
                if success:
                    log(self.serveur, "priv", auteur, " ".join(message) + "[successful]")
                    self.users.add(auteur, info["idbde"])
                    serv.privmsg(auteur, "Pseudo enregistré.")
                else:
                    log(self.serveur, "priv", auteur, " ".join(message) + "[failed]")
                    serv.privmsg(auteur, "Mot de passe invalide. (ou serveur down)")
            else:
                serv.privmsg(auteur, "Syntaxe : IDENTIFY [<username> <password>]")
        elif cmd == u"drop":
            if len(message) > 1:
                if self.users.has(auteur):
                    idbde = self.users[auteur].idbde
                    password = " ".join(message[1:])
                    success, _, _ = self.new_connection_NK(serv, "#%s" % idbde, password)
                    if success:
                        self.users.drop(idbde)
                        log(self.serveur, "priv", auteur, " ".join(message) + "[successful]")
                        serv.privmsg(auteur, "Pseudo oublié.")
                    else:
                        log(self.serveur, "priv", auteur, " ".join(message) + "[failed]")
                        serv.privmsg(auteur, "Mot de passe invalide. (ou serveur down)")
                else:
                    serv.privmsg(auteur, "Je ne connais pas votre note.")
            else:
                serv.privmsg(auteur, "Syntaxe : DROP <password>")
        elif cmd == u"join":
            if auteur in self.ops:
                if len(message) > 1:
                    if message[1] in self.chanlist:
                        serv.privmsg(auteur, (u"Je suis déjà sur %s" % (message[1])).encode("utf-8"))
                    else:
                        serv.join(message[1])
                        self.chanlist.append(message[1])
                        serv.privmsg(auteur, "Channels : " + " ".join(self.chanlist))
                        log(self.serveur, "priv", auteur, " ".join(message))
                else:
                    serv.privmsg(auteur, "Channels : " + " ".join(self.chanlist))
            else:
                notunderstood = True
        elif cmd == u"leave":
            if auteur in self.ops and len(message) > 1:
                if message[1] in self.chanlist:
                    if not (message[1] in self.stay_channels) or auteur in self.overops:
                        self.quitter(message[1].encode("utf-8"), " ".join(message[2:]))
                        self.chanlist.remove(message[1])
                        log(self.serveur, "priv", auteur, " ".join(message) + "[successful]")
                    else:
                        serv.privmsg(auteur, "Non, je reste !")
                        log(self.serveur, "priv", auteur, " ".join(message) + "[failed]")
                else:
                    serv.privmsg(auteur, "Je ne suis pas sur %s" % (message[1]))
            else:
                notunderstood = True
        elif cmd == u"stay":
            if auteur in self.overops:
                if len(message) > 1:
                    if message[1] in self.stay_channels:
                        log(self.serveur, "priv", auteur, " ".join(message) + "[failed]")
                        serv.privmsg(auteur, "Je stay déjà sur %s." % (message[1]))
                    else:
                        log(self.serveur, "priv", auteur, " ".join(message) + "[successful]")
                        self.stay_channels.append(message[1])
                        serv.privmsg(auteur, "Stay channels : " + " ".join(self.stay_channels))
                else:
                    serv.privmsg(auteur, "Stay channels : " + " ".join(self.stay_channels))
            else:
                notunderstood = True
        elif cmd == u"nostay":
            if auteur in self.overops:
                if len(message) > 1:
                    if message[1] in self.stay_channels:
                        log(self.serveur, "priv", auteur, " ".join(message) + "[successful]")
                        self.stay_channels.remove(message[1])
                        serv.privmsg(auteur, "Stay channels : " + " ".join(self.stay_channels))
                    else:
                        log(self.serveur, "priv", auteur, " ".join(message) + "[failed]")
                        serv.privmsg(auteur, "Je ne stay pas sur %s." % (message[1]))
            else:
                notunderstood = True
        elif cmd == u"die":
            if auteur in self.overops:
                log(self.serveur, "priv", auteur, " ".join(message) + "[successful]")
                self.mourir()
            else:
                notunderstood = True
        elif cmd == u"crash":
            if auteur in self.overops:
                log(self.serveur, "priv", auteur, " ".join(message) + "[successful]")
                self.crash(auteur, "priv")
            else:
                notunderstood = True
        elif cmd == u"reload":
            if auteur in self.ops:
                self.execute_something("reload", {"auteur" : auteur}, place=auteur, auteur=auteur)
            else:
                notunderstood = True
        elif cmd == u"reconnect":
            if auteur in self.ops:
                try:
                    self.nk = self.new_connection_NK(serv, config.note_pseudo,
                                            config.note_password, "special")[2]
                except Exception as exc:
                    self.nk = None
                    log(self.serveur, 'Erreur dans on_pubmsg/"cmd in ["reconnect"]\n' + str(exc))
                if self.nk != None:
                    serv.privmsg(auteur, "%s: done" % (auteur))
                    log(self.serveur, "priv", auteur, " ".join(message) + "[successful]")
                else:
                    serv.privmsg(auteur, "%s: failed" % (auteur))
                    log(self.serveur, "priv", auteur, " ".join(message) + "[failed]")
                    for report in self.report_bugs_to:
                        serv.privmsg(report, "Connection to NK2015 failed, invalid password ? Server dead ?")
            else:
                notunderstood = True
        elif cmd == u"quiet":
            if auteur in self.ops:
                if len(message) > 1:
                    if message[1] in self.quiet_channels:
                        serv.privmsg(auteur, "Je me la ferme déjà sur %s" % (message[1]))
                        log(self.serveur, "priv", auteur, " ".join(message) + "[failed]")
                    else:
                        self.quiet_channels.append(message[1])
                        serv.privmsg(auteur, "Quiet channels : " + " ".join(self.quiet_channels))
                        log(self.serveur, "priv", auteur, " ".join(message) + "[successful]")
                else:
                    serv.privmsg(auteur, "Quiet channels : " + " ".join(self.quiet_channels))
            else:
                notunderstood = True
        elif cmd == u"noquiet":
            if auteur in self.ops:
                if len(message) > 1:
                    if message[1] in self.quiet_channels:
                        self.quiet_channels.remove(message[1])
                        serv.privmsg(auteur, "Quiet channels : " + " ".join(self.quiet_channels))
                        log(self.serveur, "priv", auteur, " ".join(message) + "[successful]")
                    else:
                        serv.privmsg(auteur, "Je ne me la ferme pas sur %s." % (message[1]))
                        log(self.serveur, "priv", auteur, " ".join(message) + "[failed]")
            else:
                notunderstood = True
        elif cmd == u"say":
            if auteur in self.overops and len(message) > 2:
                serv.privmsg(message[1].encode("utf-8"), (u" ".join(message[2:])).encode("utf-8"))
                log(self.serveur, "priv", auteur, " ".join(message))
            elif len(message) <= 2:
                serv.privmsg(auteur, "Syntaxe : SAY <channel> <message>")
            else:
                notunderstood = True
        elif cmd == u"do":
            if auteur in self.overops and len(message) > 2:
                serv.action(message[1], " ".join(message[2:]))
                log(self.serveur, "priv", auteur, " ".join(message))
            elif len(message) <= 2:
                serv.privmsg(auteur, "Syntaxe : DO <channel> <action>")
            else:
                notunderstood = True
        elif cmd == u"kick":
            if auteur in self.overops and len(message) > 2:
                serv.kick(message[1].encode("utf-8"), message[2].encode("utf-8"), " ".join(message[3:]).encode("utf-8"))
                log(self.serveur, "priv", auteur, " ".join(message))
            elif len(message) <= 2:
                serv.privmsg(auteur, "Syntaxe : KICK <channel> <pseudo> [<raison>]")
            else:
                notunderstood = True
        elif cmd == u"lost":
            if auteur in self.ops and len(message) > 1:
                serv.privmsg(message[1], "J'ai perdu !")
                log(self.serveur, "priv", auteur, " ".join(message))
            elif len(message) <= 1:
                serv.privmsg(auteur, "Syntaxe : LOST <channel>")
            else:
                notunderstood = True
        elif cmd == u"solde":
            if len(message) == 1:
                if self.users.has(auteur):
                    success, solde, pseudo = nk.get_solde(self.nk, self.users[auteur].idbde, serv, auteur)
                    if success:
                        serv.privmsg(auteur, "%.2f (%s)" % (solde/100.0, pseudo.encode("utf8")))
                    log(self.serveur, "priv", auteur, " ".join(message) + ("[successful]" if success else "[failed]"))
                else:
                    serv.privmsg(auteur, "Je ne connais pas votre note.")
        elif cmd == u"ops":
            if auteur in self.overops:
                serv.privmsg(auteur, " ".join(self.ops))
            else:
                notunderstood = True
        elif cmd == u"overops":
            if auteur in self.overops:
                serv.privmsg(auteur, " ".join(self.overops))
            else:
                notunderstood = True
        elif cmd == u"whois":
            if auteur in self.ops and len(message) > 1:
                self.whois(message[1], askedwhy="cmd WHOIS", askedby=auteur)
            else:
                notunderstood = True
        else:
            notunderstood = True
        if notunderstood:
            serv.privmsg(auteur, "Je n'ai pas compris. Essayez HELP…")
    
    def on_pubmsg(self, serv, ev):
        """À la réception d'un message sur un channel."""
        if ignore_event(serv, ev):
            return
        auteur = irclib.nm_to_n(ev.source())
        canal = ev.target()
        message = ev.arguments()[0]
        try:
            message = bot_unicode(message)
        except errors.UnicodeBotError:
            if config.utf8_trigger and not canal in self.quiet_channels:
                serv.privmsg(canal, (u"%s: %s"% ( auteur, random.choice(config.utf8_fail_answers))).encode("utf8"))
            return
        pour_moi, message = self.pourmoi(serv, message)
        if pour_moi and message.split()!=[]:
            cmd = message.split()[0].lower()
            try:
                args = " ".join(message.split()[1:])
            except:
                args = ""
            if cmd in [u"meurs", u"die", u"crève"]:
                if auteur in self.overops:
                    log(self.serveur, canal, auteur, message + "[successful]")
                    self.mourir()
                else:
                    serv.privmsg(canal,(u"%s: %s"%(auteur, random.choice(config.quit_fail_messages))).encode("utf8"))
                    log(self.serveur, canal, auteur, message + "[failed]")
            elif cmd == u"reload":
                if auteur in self.ops:
                    self.execute_something("reload", {"auteur" : auteur}, place=canal, auteur=auteur)
            elif cmd == u"crash":
                if auteur in self.overops:
                    self.crash(auteur, canal)
            elif cmd in [u"part", u"leave", u"dégage", u"va-t-en", u"tut'tiresailleurs,c'estmesgalets"]:
                if auteur in self.ops and (not (canal in self.stay_channels)
                                           or auteur in self.overops):
                    self.quitter(canal)
                    log(self.serveur, canal, auteur, message + "[successful]")
                    if canal in self.chanlist:
                        self.chanlist.remove(canal)
                else:
                    serv.privmsg(canal,(u"%s: %s" % (auteur, random.choice(config.leave_fail_messages))).encode("utf8"))
                    log(self.serveur, canal, auteur, message + "[failed]")
            
            elif cmd == u"reconnect":
                if auteur in self.ops:
                    try:
                        self.nk = self.new_connection_NK(serv, config.note_pseudo,
                                              config.note_password, "special")[2]
                    except Exception as exc:
                        self.nk = None
                        log(self.serveur, 'Erreur dans on_pubmsg/"cmd in ["reconnect"]\n' + str(exc))
                    if self.nk != None:
                        serv.privmsg(canal, "%s: done" % (auteur))
                        log(self.serveur, canal, auteur, message + "[successful]")
                    else:
                        serv.privmsg(canal, "%s: failed" % (auteur))
                        log(self.serveur, canal, auteur, message + "[failed]")
                        for report in self.report_bugs_to:
                            serv.privmsg(report, "Connection to NK2015 failed, invalid password ? Server dead ?")
                else:
                    serv.privmsg(canal, "%s: %s" % (auteur, random.choice(config.pas_programme_pour_tobeir).encode("utf8")))
                    log(self.serveur, canal, auteur, message + "[failed]")

            elif cmd in [u"deviens", u"pseudo"]:
                if auteur in self.ops:
                    become = args
                    serv.nick(become)
                    log(self.serveur, canal, auteur, message + "[successful]")
    
            if cmd in [u"meur", u"meurt", u"meurre", u"meurres"] and not canal in self.quiet_channels:
                serv.privmsg(canal, '%s: Mourir, impératif, 2ème personne du singulier : "meurs" (de rien)' % (auteur))
            elif cmd in [u"ping"] and not canal in self.quiet_channels:
                serv.privmsg(canal, "%s: pong" % (auteur))

            elif cmd in [u"solde", u"!solde", u"!coca"] or cmd.startswith("!"):
                if self.users.has(auteur):
                    idbde = self.users[auteur].idbde
                    if cmd in [u"solde", u"!solde"]:
                        success, solde, pseudo = nk.get_solde(self.nk, idbde, serv, canal)
                        if success:
                            serv.privmsg(canal, "%s: %s (%s)" % (auteur, float(solde)/100, pseudo.encode("utf8")))
                    elif cmd in [u"!coca"] or cmd.startswith("!"):
                        success = nk.consomme(self.nk, idbde, message[1:], serv, canal)
                    log(self.serveur, canal, auteur, message + ("[successful]" if success else "[failed]"))
                else:
                    serv.privmsg(canal, "%s: Je ne connais pas votre pseudo note." % (auteur))
                    log(self.serveur, canal, auteur, message + "[unknown]")
            elif (re.match("(pain au chocolat|chocolatine)", message.lower())
                 and not canal in self.quiet_channels):
                serv.action(canal, "sert un pain au chocolat à %s" % (auteur))
            elif re.match("manzana",message.lower()) and not canal in self.quiet_channels:
                if auteur in config.manzana:
                    serv.action(canal, "sert une bouteille de manzana à %s" % (auteur))
                elif auteur in config.manzana_bis:
                    serv.action(canal, "sert un grand verre de jus de pomme à %s : tout le monde sait qu'il ne boit pas." % (auteur))
                else:
                    serv.action(canal, "sert un verre de manzana à %s" % (auteur))
            if isit.is_insult(message) and not canal in self.quiet_channels:
                if isit.is_not_insult(message):
                    answer = random.choice(config.compliment_answers)
                    for ligne in answer.split("\n"):
                        serv.privmsg(canal, "%s: %s" % (auteur, ligne.encode("utf8")))
                else:
                    answer = random.choice(config.insultes_answers)
                    for ligne in answer.split("\n"):
                        serv.privmsg(canal, "%s: %s" % (auteur, ligne.encode("utf8")))
            elif isit.is_compliment(message) and not canal in self.quiet_channels:
                answer = random.choice(config.compliment_answers)
                for ligne in answer.split("\n"):
                    serv.privmsg(canal, "%s: %s" % (auteur,ligne.encode("utf8")))
            gros_match = isit.is_gros(message)
            if gros_match and not canal in self.quiet_channels:
                taille = get_filesize()
                answer = u"Mais non, je ne suis pas %s, %sKo tout au plus…" % (gros_match.groups()[0], taille)
                serv.privmsg(canal, "%s: %s"%(auteur, answer.encode("utf8")))
            if isit.is_tesla(message) and not canal in self.quiet_channels:
                l1, l2 = config.tesla_answers, config.tesla_actions
                n1, n2 = len(l1), len(l2)
                i = random.randrange(n1 + n2)
                if i >= n1:
                    serv.action(canal, l2[i - n1].encode("utf8"))
                else:
                    serv.privmsg(canal, "%s: %s" % (auteur, l1[i].encode("utf8")))
            if isit.is_tamere(message) and not canal in self.quiet_channels:
                answer = random.choice(config.tamere_answers)
                for ligne in answer.split("\n"):
                    serv.privmsg(canal, "%s: %s"%(auteur, ligne.encode("utf8")))
            if isit.is_tag(message) and not canal in self.quiet_channels:
                if auteur in self.ops:
                    action = random.choice(config.tag_actions)
                    serv.action(canal, action.encode("utf8"))
                    self.quiet_channels.append(canal)
                else:
                    answer = random.choice(config.tag_answers)
                    for ligne in answer.split("\n"):
                        serv.privmsg(canal, "%s: %s" % (auteur, ligne.encode("utf8")))
            if isit.is_merci(message):
                answer = random.choice(config.merci_answers)
                for ligne in answer.split("\n"):
                    serv.privmsg(canal, "%s: %s"%(auteur, ligne.encode("utf8")))
            out = re.match(ur"^([A-Z[]|\\|[0-9]+|(¹|²|³|⁴|⁵|⁶|⁷|⁸|⁹|⁰)+)(?:| \?| !)$", message.upper())
            if re.match("ma bite dans ton oreille", message) and not canal in self.quiet_channels:
                serv.privmsg(canal, "%s: Seul un olasd peut imiter un olasd dans un de ses grands jours !" % (auteur))
            if out and not canal in self.quiet_channels:
                out = out.groups()[0]
                try:
                    iout = int(out)
                    serv.privmsg(canal, "%s: %s !" % (auteur, iout + 1))
                    if iout == 2147483647:
                        serv.privmsg(canal, "%s: Ciel, un maxint ! Heureusement que je suis en python…" % (auteur))
                        return
                    if iout + 1 > 1000 and random.randrange(4) == 0:
                        serv.privmsg(canal, "%s: Vous savez, moi et les chiffres…" % (auteur))
                        return
                except Exception as exc:
                    pass
                if re.match("[A-Y]", out):
                    alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                    serv.privmsg(canal, "%s: %s !"%(auteur, alphabet[alphabet.index(out) + 1]))
                elif out == "Z":
                    serv.privmsg(canal, "%s: Je ne vous remercie pas, j'ai l'air idiot ainsi… [ ?" % (auteur))
                elif out in "[\\":
                    serv.privmsg(canal, "%s: Nous devrions nous en tenir là, ça va finir par poser des problèmes…" % (auteur))
                elif re.match(ur"(¹|²|³|⁴|⁵|⁶|⁷|⁸|⁹|⁰)+", out):
                    def translate(mess):
                        return "".join([{u"⁰¹²³⁴⁵⁶⁷⁸⁹0123456789"[i]:u"0123456789⁰¹²³⁴⁵⁶⁷⁸⁹"[i]
                                        for i in range(20)}[j]
                                       for j in mess])
                    out = int(translate(out))
                    serv.privmsg(canal,"%s: %s !" % (auteur, translate(str(out + 1)).encode("utf8")))
            if isit.is_bonjour(message) and not canal in self.quiet_channels:
                if isit.is_night():
                    answer = random.choice(config.night_answers)
                elif isit.is_day():
                    answer = random.choice(config.bonjour_answers)
                else:
                    answer = random.choice(config.bonsoir_answers)
                serv.privmsg(canal, answer.format(auteur).encode("utf8"))
            if isit.is_bonne_nuit(message) and not canal in self.quiet_channels:
                answer = random.choice(config.bonne_nuit_answers)
                serv.privmsg(canal, answer.format(auteur).encode("utf8"))
            if isit.is_pan(message) and not canal in self.quiet_channels:
                serv.privmsg(canal, "%s: ce n'est pas sur moi qu'il faut tirer, même si je sais que j'attire l'œil !" % (auteur))
        else:
            if message in [u"!pain au chocolat", u"!chocolatine"] and not canal in self.quiet_channels:
                serv.action(canal, "sert un pain au chocolat à %s" % (auteur))
            if message in [u"!manzana"] and not canal in self.quiet_channels:
                if auteur in config.manzana:
                    serv.action(canal, "sert une bouteille de manzana à %s" % (auteur))
                elif auteur in config.manzana_bis:
                    serv.action(canal, "sert un grand verre de jus de pomme à %s : tout le monde sait qu'il ne boit pas." % (auteur))
                else:
                    serv.action(canal, "sert un verre de manzana à %s" % (auteur))
            if re.match(config.buffer_fail_regexp, message, flags=re.UNICODE) and not canal in self.quiet_channels:
                failanswers = config.buffer_fail_answers
                answer = random.choice(failanswers)
                serv.privmsg(canal, ("%s: %s"%(auteur,answer)).encode("utf8"))
            if not canal in self.quiet_channels:
                mypseudo = self.nick
                if re.match((u"^(" + u"|".join(config.bonjour_triggers)
                                   + ur")( {}| all| tout le monde| (à )?tous)(\.| ?!)?$"
                             ).format(mypseudo).lower(), message.strip().lower()):
                    answer = random.choice(config.bonjour_answers)
                    serv.privmsg(canal, answer.format(auteur).encode("utf8"))
        if (isit.is_perdu(message) and not canal in self.quiet_channels):
            # proba de perdre sur trigger :
            #  avant 30min (enfin, config) : 0
            #  ensuite, +25%/30min, linéairement
            deltat = time.time() - self.last_perdu
            barre = (deltat - config.time_between_perdu)/(2*3600.0)
            if random.uniform(0, 1) < barre:
                serv.privmsg(canal, "%s: J'ai perdu !" % (auteur))
                self.last_perdu = time.time()

    def on_action(self, serv, ev):
        """À la réception d'une action."""
        if ignore_event(serv, ev):
            return
        action = ev.arguments()[0]
        auteur = irclib.nm_to_n(ev.source())
        channel = ev.target()
        try:
            action = bot_unicode(action)
        except errors.UnicodeBotError:
            if config.utf8_trigger and not channel in self.quiet_channels:
                serv.privmsg(channel, (u"%s: %s"%(auteur,random.choice(config.utf8_fail_answers))).encode("utf8"))
            return
        mypseudo = self.nick
        
        if isit.is_bad_action_trigger(action, mypseudo) and not channel in self.quiet_channels:
            l1, l2 = config.bad_action_answers, config.bad_action_actions
            n1, n2 = len(l1), len(l2)
            i = random.randrange(n1 + n2)
            if i >= n1:
                serv.action(channel, l2[i - n1].format(auteur).encode("utf8"))
            else:
                serv.privmsg(channel, l1[i].format(auteur).encode("utf8"))
        if isit.is_good_action_trigger(action, mypseudo) and not channel in self.quiet_channels:
            l1, l2 = config.good_action_answers, config.good_action_actions
            n1, n2 = len(l1), len(l2)
            i = random.randrange(n1 + n2)
            if i >= n1:
                serv.action(channel, l2[i-n1].format(auteur).encode("utf8"))
            else:
                serv.privmsg(channel, l1[i].format(auteur).encode("utf8"))
    
    def on_kick(self, serv, ev):
        """À la réception d'une action."""
        auteur = irclib.nm_to_n(ev.source())
        channel = ev.target()
        victime = ev.arguments()[0]
        raison = ev.arguments()[1]
        if victime == self.nick:
            log(self.serveur, ("%s kické de %s par %s (raison : %s)" % (victime, channel, auteur, raison)).decode("utf-8"))
            time.sleep(2)
            serv.join(channel)
            l1, l2 = config.kick_answers, config.kick_actions
            n1, n2 = len(l1), len(l2)
            i = random.randrange(n1 + n2)
            if i >= n1:
                serv.action(channel, l2[i-n1].format(auteur).encode("utf8"))
            else:
                serv.privmsg(channel, l1[i].format(auteur).encode("utf8"))
    
    ### .fork trick
    def start_as_daemon(self, outfile):
        sys.stderr = Logger(outfile)
        self.start()
    

class Logger(object):
    """Pour écrire ailleurs que sur stdout"""
    def __init__(self, filename="basile.full.log"):
        self.filename = filename

    def write(self, message):
        f = open(self.filename, "a")
        f.write(message)
        f.close()

def main():
    """Exécution principale : lecture des paramètres et lancement du bot."""
    if len(sys.argv) == 1:
        print "Usage : basile.py <serveur> [--debug] [--no-output] [--daemon [--pidfile]] [--outfile]"
        print "        --outfile sans --no-output ni --daemon n'a aucun effet"
        exit(1)
    serveur = sys.argv[1]
    if "--daemon" in sys.argv:
        thisfile = os.path.realpath(__file__)
        thisdirectory = thisfile.rsplit("/", 1)[0]
        os.chdir(thisdirectory)
        daemon = True
    else:
        daemon = False
    if "debug" in sys.argv or "--debug" in sys.argv:
        debug = True
    else:
        debug = False
    if "--quiet" in sys.argv:
        config.debug_stdout = False
    serveurs = {"a♡" : "acoeur.crans.org",
              "acoeur" : "acoeur.crans.org",
              "acoeur.crans.org" : "acoeur.crans.org",
              "irc" : "irc.crans.org",
              "crans" : "irc.crans.org",
              "irc.crans.org" : "irc.crans.org"}
    if "--no-output" in sys.argv or "--daemon" in sys.argv:
        outfile = "/var/log/bots/basile.full.log"
        for arg in sys.argv:
            arg = arg.split("=")
            if arg[0].strip('-') in ["out", "outfile", "logfile"]:
                outfile = arg[1]
        sys.stdout = Logger(outfile)
    try:
        serveur = serveurs[serveur]
    except KeyError:
        print "Server Unknown : %s" % (serveur)
        exit(404)
    basile = Basile(serveur,debug)
    # Si on reçoit un SIGHUP, on reload la config
    def sighup_handler(signum, frame):
        basile.execute_reload(auteur="SIGHUP")
    signal.signal(signal.SIGHUP, sighup_handler)
    # Daemonization
    if daemon:
        child_pid = os.fork()
        if child_pid == 0:
            os.setsid()
            basile.start_as_daemon(outfile)
        else:
            # on enregistre le pid de basile
            pidfile = "/var/run/bots/basile.pid"
            for arg in sys.argv:
                arg = arg.split("=")
                if arg[0].strip('-') in ["pidfile"]:
                    pidfile = arg[1]
            f = open(pidfile, "w")
            f.write("%s\n" % child_pid)
            f.close()
    else:
        basile.start()

if __name__ == "__main__":
    main()
