#!/usr/bin/python
# -*- coding:utf8 -*-

""" Des fonctions utilitaire pour savoir si on est dans tel ou tel cas. """

import re
import time

import config

def regex_join(liste, avant=u".*(?:^| )", apres=u"(?:$|\.| |,|;).*"):
    """Fabrique une regexp à partir d'une liste d'éléments à matcher."""
    return avant + u"(" + u"|".join(liste) + u")" + apres

def is_something(chain, regexp=None, matches=[], avant=u".*(?:^| )", apres=u"(?:$|\.| |,|;).*",
                 case_sensitive=False):
    """Vérifie si chain contient un des éléments de ``matches``.
       Si ``regexp`` est fournie, c'est simplement elle qui est testée"""
    if not case_sensitive:
        chain = chain.lower()
        apres = apres.lower()
        avant = avant.lower()
    if regexp == None:
        regexp = regex_join(matches, avant, apres)
        regexp = re.compile(regexp)
    o = regexp.match(chain)
    return o

def regexp_compile():
    """Compilation des regexp à partir de la conf.
       Place les résultats dans le namespace de ``config``"""
    config.insult_regexp = regex_join(config.insultes, avant=u".*(?:^| |')")
    config.insult_regexp_compiled = re.compile(config.insult_regexp)
    
    config.not_insult_regexp = u".*pas %s%s" % (config.amplifier_regexp, config.insult_regexp)
    config.not_insult_regexp_compiled = re.compile(config.not_insult_regexp)
    
    config.compliment_regexp = regex_join(config.compliment_triggers, avant=u".*(?:^| |')")
    config.compliment_regexp_compiled = re.compile(config.compliment_regexp)
    
    config.perdu_regexp = regex_join(config.perdu)
    config.perdu_regexp_compiled = re.compile(config.perdu_regexp)
    
    config.tag_regexp = regex_join(config.tag_triggers)
    config.tag_regexp_compiled = re.compile(config.tag_regexp)
    
    config.gros_regexp = regex_join(config.gros)
    config.gros_regexp_compiled = re.compile(config.gros_regexp)
    
    config.tesla_regexp = regex_join(config.tesla_triggers, avant=u"^", apres="$")
    config.tesla_regexp_compiled = re.compile(config.tesla_regexp)
    
    config.merci_regexp = regex_join(config.merci_triggers)
    config.merci_regexp_compiled = re.compile(config.merci_regexp)
    
    config.tamere_regexp = regex_join(config.tamere_triggers)
    config.tamere_regexp_compiled = re.compile(config.tamere_regexp)
    
    config.bonjour_regexp = regex_join(config.bonjour_triggers, avant=u"^")
    config.bonjour_regexp_compiled = re.compile(config.bonjour_regexp)
    
    config.bonne_nuit_regexp = regex_join(config.bonne_nuit_triggers, avant=u"^")
    config.bonne_nuit_regexp_compiled = re.compile(config.bonne_nuit_regexp)
    
    config.pan_regexp = regex_join(config.pan_triggers, avant=".*", apres=".*")
    config.pan_regexp_compiled = re.compile(config.pan_regexp)
    
    
regexp_compile()
def is_insult(chain, debug=True):
    """Vérifie si ``chain`` contient une insulte."""
    return is_something(chain, config.insult_regexp_compiled)
def is_not_insult(chain):
    """Vérifie si ``chain`` contient une insulte à la forme négative."""
    return is_something(chain, config.not_insult_regexp_compiled)
def is_compliment(chain, debug=True):
    """Vérifie si ``chain`` contient un compliment."""
    return is_something(chain, config.compliment_regexp_compiled)
def is_perdu(chain):
    """Vérifie si ``chain`` contient une raison de perdre."""
    return is_something(chain, config.perdu_regexp_compiled)
def is_tag(chain):
    """Vérifie si ``chain`` demande de fermer sa gueule."""
    return is_something(chain, config.tag_regexp_compiled)
def is_gros(chain):
    """Vérifie si ``chain`` traite de gros."""
    return is_something(chain, config.gros_regexp_compiled)
def is_tesla(chain):
    """Vérifie si ``chain`` est un ping."""
    return is_something(chain, config.tesla_regexp_compiled)
def is_merci(chain):
    """Vérifie si ``chain`` contient un remerciement."""
    return is_something(chain, config.merci_regexp_compiled)
def is_tamere(chain):
    """Vérifie si ``chain`` traite ma mère."""
    return is_something(chain, config.tamere_regexp_compiled)
def is_bad_action_trigger(chain,pseudo):
    """Vérifie si ``chain`` est une action méchante.
       On a besoin d'une regexp dynamique à cause du pseudo qui peut changer."""
    return is_something(chain, matches=config.bad_action_triggers, avant=u"^",
                            apres="(?: [a-z]*ment)? %s($|\.| |,|;).*" % (pseudo))
def is_good_action_trigger(chain,pseudo):
    """Vérifie si ``chain`` est une action gentille.
       On a besoin d'une regexp dynamique à cause du pseudo qui peut changer."""
    return is_something(chain, matches=config.good_action_triggers, avant=u"^",
                            apres="(?: [a-z]*ment)? %s($|\.| |,|;).*" % (pseudo))
def is_bonjour(chain):
    """Vérifie si ``chain`` contient un bonjour."""
    return is_something(chain, config.bonjour_regexp_compiled)
def is_bonne_nuit(chain):
    """Vérifie si ``chain`` contient un bonne nuit."""
    return is_something(chain, config.bonne_nuit_regexp_compiled)
def is_pan(chain):
    """Vérifie si ``chain`` contient un pan."""
    return is_something(chain, config.pan_regexp_compiled)

def is_time(conf):
    """Vérifie si l'heure actuelle est entre les deux heures ``conf[0]`` et ``conf[1]``"""
    _, _, _, h, m, s, _, _, _ = time.localtime()
    return (conf[0], 0, 0) < (h, m, s) < (conf[1], 0, 0)
def is_day():
    """Vérifie si on est le jour."""
    return is_time(config.daytime)
def is_night():
    """Vérifie si on est la nuit."""
    return is_time(config.nighttime)
